@@ -2,8 +2,10 @@
 #of this repository contains the full copyright notices and license terms.
 "Currency"
 from trytond.model import ModelView, ModelSQL, fields
+from trytond.tools import safe_eval
 import time
-from decimal import Decimal
+from decimal import Decimal, ROUND_HALF_EVEN
+import datetime
 
 
 class Currency(ModelSQL, ModelView):
@@ -13,10 +15,11 @@ class Currency(ModelSQL, ModelView):
     name = fields.Char('Name', required=True, translate=True)
     symbol = fields.Char('Symbol', size=10, required=True)
     code = fields.Char('Code', size=3, required=True)
-    rate = fields.Function('get_rate', string='Current rate', digits=(12, 6))
+    rate = fields.Function('get_rate', type='numeric', string='Current rate',
+            digits=(12, 6), on_change_with=['rates'])
     rates = fields.One2Many('currency.currency.rate', 'currency', 'Rates')
     rounding = fields.Numeric('Rounding factor', digits=(12, 6), required=True)
-    digits = fields.Integer('Diplay Digits')
+    digits = fields.Integer('Display Digits')
     active = fields.Boolean('Active')
 
     # monetary formatting
@@ -42,6 +45,9 @@ class Currency(ModelSQL, ModelView):
             'no_rate': 'No rate found for the currency: %s at the date: %s',
             'invalid_grouping': 'Invalid Grouping!',
             })
+        self._rpc.update({
+            'compute': False,
+            })
 
     def default_active(self, cursor, user, context=None):
         return True
@@ -91,7 +97,7 @@ class Currency(ModelSQL, ModelView):
         '''
         for currency in self.browse(cursor, user, ids):
             try:
-                grouping = eval(currency.mon_grouping)
+                grouping = safe_eval(currency.mon_grouping)
                 for i in grouping:
                     if not isinstance(i, int):
                         return False
@@ -115,6 +121,18 @@ class Currency(ModelSQL, ModelView):
             i += 1
         return args2
 
+    def on_change_with_rate(self, cursor, user, ids, vals, context=None):
+        now = datetime.date.today()
+        closer = datetime.date.min
+        res = Decimal('0.0')
+        for rate in vals.get('rates', []):
+            if 'date' not in rate or 'rate' not in rate:
+                continue
+            if rate['date'] <= now and rate['date'] > closer:
+                res = rate['rate']
+                closer = rate['date']
+        return res
+
     def get_rate(self, cursor, user, ids, name, arg, context=None):
         '''
         Return the rate at the date from the context or the current date
@@ -146,9 +164,19 @@ class Currency(ModelSQL, ModelView):
                 res[currency_id] = id2rate[res[currency_id]].rate
         return res
 
-    def round(self, cursor, user, currency, amount):
-        'Round the amount depending of the currency'
-        return (amount / currency.rounding).quantize(Decimal('1.')) * currency.rounding
+    def round(self, cursor, user, currency, amount, rounding=ROUND_HALF_EVEN):
+        '''
+        Round the amount depending of the currency
+
+        :param cursor: the database cursor
+        :param user: the user id
+        :param currency: a BrowseRecord of currency.currency
+        :param amout: a Decimal
+        :param rounding: the rounding option
+        :return: a Decimal
+        '''
+        return (amount / currency.rounding).quantize(Decimal('1.'),
+                rounding=rounding) * currency.rounding
 
     def is_zero(self, cursor, user, currency, amount):
         'Return True if the amount can be considered as zero for the currency'
@@ -209,7 +237,8 @@ class Rate(ModelSQL, ModelView):
     _rec_name = 'date'
     date = fields.Date('Date', required=True, select=1)
     rate = fields.Numeric('Rate', digits=(12, 6), required=1)
-    currency = fields.Many2One('currency.currency', 'Currency')
+    currency = fields.Many2One('currency.currency', 'Currency',
+            ondelete='CASCADE',)
 
     def __init__(self):
         super(Rate, self).__init__()
